DATA=/mnt/data/pgdata

function initcluster() {
	echo "======== init $1 ========"
	killall postgres >> keybench.log 2>&1
	rm -Rf $DATA >> keybench.log 2>&1
	pg_ctl -D $DATA init >> keybench.log 2>&1
	pg_config >> keybench.log 2>&1
	cp postgresql.conf $DATA
	pg_ctl -D $DATA -l bench.log start >> keybench.log 2>&1
	sleep 5
	createdb test >> keybench.log 2>&1
}

function init() {
	echo "======== scale $1 ========"
	psql -h localhost test -c "drop table if exists stuff_text" >> keybench.log 2>&1
	psql -h localhost test -c "drop table if exists stuff_numeric" >> keybench.log 2>&1
	psql -h localhost test -c "drop table if exists stuff_float" >> keybench.log 2>&1
	psql -h localhost test -c "drop table if exists stuff_int" >> keybench.log 2>&1

	psql -h localhost test -c "create table stuff_text    as select (random())::text         as randtxt   from generate_series(1,$1)" >> keybench.log 2>&1
	psql -h localhost test -c "create table stuff_numeric as select (random())::numeric      as randnum   from generate_series(1,$1)" >> keybench.log 2>&1
	psql -h localhost test -c "create table stuff_float   as select (random())::float        as randfloat from generate_series(1,$1)" >> keybench.log 2>&1
	psql -h localhost test -c "create table stuff_int     as select (10000000*random())::int as randint   from generate_series(1,$1)" >> keybench.log 2>&1

        psql -h localhost test -c "drop table if exists stuff_text_asc" >> keybench.log 2>&1
        psql -h localhost test -c "drop table if exists stuff_numeric_asc" >> keybench.log 2>&1
        psql -h localhost test -c "drop table if exists stuff_float_asc" >> keybench.log 2>&1
        psql -h localhost test -c "drop table if exists stuff_int_asc" >> keybench.log 2>&1

        psql -h localhost test -c "create table stuff_text_asc    as select (random())::text         as randtxt   from generate_series(1,$1) order by randtxt"   >> keybench.log 2>&1
        psql -h localhost test -c "create table stuff_numeric_asc as select (random())::numeric      as randnum   from generate_series(1,$1) order by randnum"   >> keybench.log 2>&1
        psql -h localhost test -c "create table stuff_float_asc   as select (random())::float        as randfloat from generate_series(1,$1) order by randfloat" >> keybench.log 2>&1
        psql -h localhost test -c "create table stuff_int_asc     as select (10000000*random())::int as randint   from generate_series(1,$1) order by randint"   >> keybench.log 2>&1

        psql -h localhost test -c "drop table if exists stuff_text_desc" >> keybench.log 2>&1
        psql -h localhost test -c "drop table if exists stuff_numeric_desc" >> keybench.log 2>&1
        psql -h localhost test -c "drop table if exists stuff_float_desc" >> keybench.log 2>&1
        psql -h localhost test -c "drop table if exists stuff_int_desc" >> keybench.log 2>&1

        psql -h localhost test -c "create table stuff_text_desc   as select (random())::text          as randtxt   from generate_series(1,$1)  order by randtxt desc"  >> keybench.log 2>&1
        psql -h localhost test -c "create table stuff_numeric_desc as select (random())::numeric      as randnum   from generate_series(1,$1) order by randnum desc"   >> keybench.log 2>&1
        psql -h localhost test -c "create table stuff_float_desc   as select (random())::float        as randfloat from generate_series(1,$1) order by randfloat desc" >> keybench.log 2>&1
        psql -h localhost test -c "create table stuff_int_desc    as select (10000000*random())::int  as randint   from generate_series(1,$1)  order by randint desc"  >> keybench.log 2>&1

	psql -h localhost test -c "analyze"               >> keybench.log 2>&1
	psql -h localhost test -c "vacuum freeze"         >> keybench.log 2>&1
	psql -h localhost test -c "checkpoint"            >> keybench.log 2>&1
}

function query() {
	echo "$1"
	for i in `seq 1 20`; do
		r=`echo "$1" | psql -h localhost test | grep 'Time'`
		echo $r
	done
}

function queries() {

        # random

	# text
	query "select percentile_disc(0) within group (order by randtxt) from stuff_text"
	query "select count(distinct randtxt) from stuff_text"
	query "select * from (select * from stuff_text order by randtxt offset 100000000000) foo"

	# numeric
	query "select percentile_disc(0) within group (order by randnum) from stuff_numeric"
	query "select count(distinct randnum) from stuff_numeric"
	query "select * from (select * from stuff_numeric order by randnum offset 100000000000) foo"

	# float
	query "select percentile_disc(0) within group (order by randfloat) from stuff_float"
	query "select count(distinct randfloat) from stuff_float"
	query "select * from (select * from stuff_float order by randfloat offset 100000000000) foo"

	# int
	query "select percentile_disc(0) within group (order by randint) from stuff_int"
	query "select count(distinct randint) from stuff_int"
	query "select * from (select * from stuff_int order by randint offset 100000000000) foo"

        # ascending

        # text
        query "select percentile_disc(0) within group (order by randtxt) from stuff_text_asc"
        query "select count(distinct randtxt) from stuff_text_asc"
        query "select * from (select * from stuff_text_asc order by randtxt offset 100000000000) foo"

        # numeric
        query "select percentile_disc(0) within group (order by randnum) from stuff_numeric_asc"
        query "select count(distinct randnum) from stuff_numeric_asc"
        query "select * from (select * from stuff_numeric_asc order by randnum offset 100000000000) foo"

        # float
        query "select percentile_disc(0) within group (order by randfloat) from stuff_float_asc"
        query "select count(distinct randfloat) from stuff_float_asc"
        query "select * from (select * from stuff_float_asc order by randfloat offset 100000000000) foo"

        # int
        query "select percentile_disc(0) within group (order by randint) from stuff_int_asc"
        query "select count(distinct randint) from stuff_int_asc"
        query "select * from (select * from stuff_int_asc order by randint offset 100000000000) foo"

        # descending

        # text
        query "select percentile_disc(0) within group (order by randtxt) from stuff_text_desc"
        query "select count(distinct randtxt) from stuff_text_desc"
        query "select * from (select * from stuff_text_desc order by randtxt offset 100000000000) foo"

        # numeric
        query "select percentile_disc(0) within group (order by randnum) from stuff_numeric_desc"
        query "select count(distinct randnum) from stuff_numeric_desc"
        query "select * from (select * from stuff_numeric_desc order by randnum offset 100000000000) foo"

        # float
        query "select percentile_disc(0) within group (order by randfloat) from stuff_float_desc"
        query "select count(distinct randfloat) from stuff_float_desc"
        query "select * from (select * from stuff_float_desc order by randfloat offset 100000000000) foo"

        # int
        query "select percentile_disc(0) within group (order by randint) from stuff_int_desc"
        query "select count(distinct randint) from stuff_int_desc"
        query "select * from (select * from stuff_int_desc order by randint offset 100000000000) foo"

}

initcluster

init 100000
queries

init 1000000
queries

init 2000000
queries

init 3000000
queries

init 4000000
queries

init 5000000
queries
